"""
Unittests for the graph module.
"""

import unittest
from graph import Node, Edge, Graph, NoPathGraphException


class TestEdge(unittest.TestCase):

    def test_basic_edges(self):
        """Test the basic edge functions."""
        node1 = Node()
        node2 = Node()
        edge1 = Edge(node2, node1)
        edge2 = Edge(node1, node2)
        self.assert_(node1._in_edges == [edge2])
        self.assert_(node1._out_edges == [edge1])
        Edge(node2, node1)
    
    def test_edge_properties(self):
        """Test changing the head or tail via the property.""" 
        node1 = Node()
        node2 = Node()
        edge1 = Edge(node1, node2)
        edge2 = Edge(node2, node1)
        edge1.head = node2
        self.assert_(node2._in_edges == [edge2, edge1])

    def test_edge_none(self):
        """Test setting the head or tail to None.""" 
        node1 = Node()
        node2 = Node()
        edge1 = Edge(node1, node2)
        edge2 = Edge(None, node1)
        edge1.tail = None
        edge1.head = None
        edge2.head = None
        edge2.tail = None
        self.assert_(node2._in_edges == [])
        self.assert_(node2._out_edges == [])


class TestGraph(unittest.TestCase):

    def test_basic_graph(self):
        """Test the basic construction of a graph."""
        node1 = Node()
        node2 = Node()
        edge1 = Edge(node2, node1)
        edge2 = Edge(node1, node2)
        graph = Graph([edge1, edge2])
        self.assert_(graph.nodes, set([node2, node1]))
        
    def test_remove_edge(self):
        """Test the basic construction of a graph."""
        node1 = Node()
        node2 = Node()
        edge1 = Edge(node2, node1)
        edge2 = Edge(node1, node2)
        graph = Graph([edge1, edge2])
        graph.remove_edge(edge1)
        self.assert_(edge1 not in graph._edges)

    def test_shortest_path(self):
        """Test the shortest path algorithm for a simple example."""
        nodes = [Node() for _ in range(5)]
        # create a diamond shape
        edges = [Edge(nodes[1], nodes[0]),
                 Edge(nodes[2], nodes[1]),
                 Edge(nodes[4], nodes[2]),
                 Edge(nodes[3], nodes[0]),
                 Edge(nodes[4], nodes[3]),
                ]
        graph = Graph(edges)
        path, length = graph.shortest_path(nodes[0], nodes[4])
        self.assert_(path == [edges[3], edges[4]])
        self.assert_(length == 2)
        self.assertRaises(NoPathGraphException,
                          lambda: graph.shortest_path(nodes[4], nodes[0]))
        
    
    # TODO: add more tests for the shortest path algorithm,
    #    especially with respect to path weight updates

    
if __name__ == '__main__':
    unittest.main()